from markdown import Markdown
from rich import print
from typing import Any, override
from pathlib import Path
from marko.inline import Link, Image
from marko.block import FencedCode
from marko.html_renderer import HTMLRenderer
from marko.parser import Parser
from zona.config import ZonaConfig
from zona.layout import Layout

from markdown.treeprocessors import Treeprocessor
from markdown.extensions.codehilite import CodeHiliteExtension
from markdown.extensions.extra import ExtraExtension
from markdown.extensions.smarty import SmartyExtension
from markdown.extensions.sane_lists import SaneListExtension
from pymdownx.inlinehilite import InlineHiliteExtension
import xml.etree.ElementTree as etree


from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer
from pygments.formatters import HtmlFormatter

from zona import util
from zona.models import Item
from zona.log import get_logger

logger = get_logger()


class ZonaLinkProcessor(Treeprocessor):
    def __init__(
        self,
        config: ZonaConfig | None,
        resolve: bool = False,
        source: Path | None = None,
        layout: Layout | None = None,
        item_map: dict[Path, Item] | None = None,
    ):
        super().__init__()
        self.resolve: bool = resolve
        if self.resolve:
            assert source is not None
            assert layout is not None
            assert item_map is not None
            self.source: Path = source.resolve()
            self.layout: Layout = layout
            self.item_map: dict[Path, Item] = item_map
            self.config: ZonaConfig | None = config

    @override
    def run(self, root: etree.Element):
        for element in root.iter("a"):
            href = element.get("href")
            if not href:
                continue
            if self.resolve:
                cur = Path(href)
                _href = href
                if href.startswith("/"):
                    # resolve relative to content root
                    resolved = (
                        self.layout.content / cur.relative_to("/")
                    ).resolve()
                else:
                    # treat as relative link and try to resolve
                    resolved = (self.source.parent / cur).resolve()
                # only substitute if link points to an actual file
                if resolved.exists():
                    item = self.item_map.get(resolved)
                    if item:
                        href = util.normalize_url(item.url)
                        element.set("href", href)
                        logger.debug(
                            f"Link in file {self.source}: {_href} resolved to {href}"
                        )
                    else:
                        logger.debug(
                            f"Warning: resolved path {resolved} not found in item map"
                        )
            element.set("target", "_blank")


def get_formatter(config: ZonaConfig):
    c = config.markdown.syntax_highlighting
    formatter = HtmlFormatter(
        style=c.theme, nowrap=not c.wrap, nobackground=True
    )
    return formatter


def md_to_html(
    content: str,
    config: ZonaConfig | None,
    resolve_links: bool = False,
    source: Path | None = None,
    layout: Layout | None = None,
    item_map: dict[Path, Item] | None = None,
) -> str:
    if config:
        md = Markdown(
            extensions=[
                CodeHiliteExtension(
                    linenums=False,
                    noclasses=False,
                    pygments_style=config.markdown.syntax_highlighting.theme,
                ),
                ExtraExtension(),
                SmartyExtension(),
                "pymdownx.tilde",
                "pymdownx.caret",
                "pymdownx.smartsymbols",
                InlineHiliteExtension(css_class="codehilite"),
                SaneListExtension(),
            ]
        )
    else:
        md = Markdown()
    if resolve_links:
        if source is None or layout is None or item_map is None:
            raise TypeError(
                "md_to_html() missing source and ctx when resolve_links is true"
            )
        md.treeprocessors.register(
            item=ZonaLinkProcessor(
                config, resolve_links, source, layout, item_map
            ),
            name="zona_links",
            priority=15,
        )
    return md.convert(content)


def get_style_defs(config: ZonaConfig) -> str:
    formatter = get_formatter(config)
    defs = formatter.get_style_defs(".codehilite")
    assert isinstance(defs, str)
    return defs
